
/*******************************************************************************

 Copyright (c) 2009, Charles McGarvey
 All rights reserved.
 
 Redistribution   and   use  in  source  and  binary  forms,  with  or  without
 modification, are permitted provided that the following conditions are met:
 
   * Redistributions  of  source  code  must retain the above copyright notice,
     this list of conditions and the following disclaimer.
   * Redistributions  in binary form must reproduce the above copyright notice,
     this  list of conditions and the following disclaimer in the documentation
     and/or other materials provided with the distribution.
 
 THIS  SOFTWARE  IS  PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 AND  ANY  EXPRESS  OR  IMPLIED  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 DISCLAIMED.  IN  NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
 FOR  ANY  DIRECT,  INDIRECT,  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 DAMAGES  (INCLUDING,  BUT  NOT  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 SERVICES;  LOSS  OF  USE,  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 CAUSED  AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 OR  TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

*******************************************************************************/

#ifndef _MOOF_DISPATCHER_HH_
#define _MOOF_DISPATCHER_HH_

#include <string>

#include <boost/bind.hpp>
#include <boost/function.hpp>
#include <boost/shared_ptr.hpp>


namespace Mf {


/**
 * Interface for a notification class.
 */

class Notification
{
public:
	virtual ~Notification();
};


/**
 * Dispatcher of notifications to interested parties.
 */

class Dispatcher
{
	class Impl;
	boost::shared_ptr<Impl> impl_;

public:

	typedef void* Handler;
	typedef boost::function<void(const Notification*)> Function;

	Dispatcher();
	~Dispatcher();

	// get global instance
	static Dispatcher& getInstance();

	Handler addHandler(const std::string& message, const Function& callback);
	Handler addHandler(const std::string& message, const Function& callback,
			Handler id);

	void removeHandler(Handler id);

	void dispatch(const std::string& message, const Notification* param = 0);
};


namespace dispatcher {

inline Dispatcher::Handler addHandler(const std::string& message,
		const Dispatcher::Function& callback)
{
	return Dispatcher::getInstance().addHandler(message, callback);
}

inline Dispatcher::Handler addHandler(const std::string& message,
		const Dispatcher::Function& callback, Dispatcher::Handler id)
{
	return Dispatcher::getInstance().addHandler(message, callback, id);
}

inline void removeHandler(Dispatcher::Handler id)
{
	Dispatcher::getInstance().removeHandler(id);
}

inline void dispatch(const std::string& message, const Notification* param = 0)
{
	Dispatcher::getInstance().dispatch(message, param);
}

} // namespace dispatch


} // namespace Mf

#endif // _MOOF_DISPATCHER_HH_

/** vim: set ts=4 sw=4 tw=80: *************************************************/

